@@ -9,6 +9,7 @@ from importlib.metadata import version, PackageNotFoundError
 from requests.exceptions import Timeout
 import sys
 import pendulum
+import re
 
 try:
     import colorama # explicitly added so for the shiv build
@@ -22,22 +23,22 @@ except ImportError:
 from pipeline.time_manager import TimeManager
 from pipeline.create_sensors_db import get_db_connection, create_packaged_db, reset_user_db # get_user_db_path, ensure_user_db, 
 from pipeline.api.eds import demo_eds_webplot_point_live, EdsClient, load_historic_data, EdsLoginException, demo_eds_save_point_export
-from pipeline import environment
+from pipeline.environment import is_termux, is_windows, is_pyz, is_elf, is_pipx, matplotlib_enabled, open_text_file_in_default_app
 from pipeline.security_and_config import get_eds_api_credentials, get_external_api_credentials, get_eds_db_credentials, get_all_configured_urls, get_configurable_plant_name, init_security, CONFIG_PATH
-from pipeline.termux_setup import setup_termux_shortcut
+from pipeline.termux_setup import setup_termux_install, cleanup_termux_install
 from pipeline.windows_setup import setup_windows_install, cleanup_windows_install
 from pipeline import helpers
 from pipeline.plotbuffer import PlotBuffer
-from pipeline.version_info import  PIP_PACKAGE_NAME, PIPELINE_VERSION, __version__, get_package_version
+from pipeline.version_info import  PIP_PACKAGE_NAME, PIPELINE_VERSION, __version__, get_package_version, get_package_name
 #from pipeline.helpers import setup_logging
 
 # --- TERMUX SETUP HOOK ---
 # This runs on every command (including --version and --help or without sub commands), 
 # but the function's internal logic
 # ensures the shortcut file is only created once in the Termux environment.
-if environment.is_termux():
-    setup_termux_shortcut()
-elif environment.is_windows():
+if is_termux():
+    setup_termux_install()
+elif is_windows():
     setup_windows_install()
 
 # -- Versioning --
@@ -88,15 +89,28 @@ def list_sensors(
         packaged_db = create_packaged_db()
         user_db = reset_user_db(packaged_db)
 
-    # db_path: str = "sensors.db"
-    if db_path is not None:
-        conn = sqlite3.connect(db_path)
-    else:  
-        conn = get_db_connection()
-    cur = conn.cursor()
-    cur.execute("SELECT idcs, iess, zd, ovation_drop, units, description FROM sensors")
-    rows = cur.fetchall()
-    conn.close()
+    try:
+        # db_path: str = "sensors.db"
+        if db_path is not None:
+            conn = sqlite3.connect(db_path)
+        else:  
+            conn = get_db_connection()
+        cur = conn.cursor()
+        cur.execute("SELECT idcs, iess, zd, ovation_drop, units, description FROM sensors")
+        rows = cur.fetchall()
+        conn.close()
+    except:
+        # if fail, it is likely the use has an outdated db on their system. Force update, then run again.
+        packaged_db = create_packaged_db()
+        user_db = reset_user_db(packaged_db)
+        if db_path is not None:
+            conn = sqlite3.connect(db_path)
+        else:  
+            conn = get_db_connection()
+        cur = conn.cursor()
+        cur.execute("SELECT idcs, iess, zd, ovation_drop, units, description FROM sensors")
+        rows = cur.fetchall()
+        conn.close()
 
     table = Table(title="Common Sensor Cheat Sheet (hard-coded)")
     table.add_column("IDCS", style="cyan")
@@ -115,13 +129,14 @@ def list_sensors(
     console.print("⚠️ The ZD for the Stiles plant is WWTF", style = "magenta")
 
 @app.command()
-def live_query(
+def live(
     idcs: list[str] = typer.Argument(..., help="Provide known idcs values that match the given zd."), # , "--idcs", "-i"
     zd: str = typer.Option('Maxson', "--zd", "-z", help = "Define the EDS ZD from your secrets file. This must correlate with your idcs point selection(s)."),
     webplot: bool = typer.Option(False,"--webplot","-w",help = "Use a web-based plot (plotly) instead of matplotlib. Useful for remote servers without display.")
 ):
     """live data plotting, based on CSV query files. Coming soon - call any, like the 'trend' command."""
-    demo_eds_webplot_point_live()
+    typer.echo(f"Coming soon!")
+    #demo_eds_webplot_point_live()
 
 @app.command()
 def defaultplant(
@@ -136,7 +151,7 @@ def trend(
     idcs: list[str] = typer.Argument(None, help="Provide known idcs values that match the given zd."), # , "--idcs", "-i"
     starttime: str = typer.Option(None, "--start", "-s", help="Identify start time. Use any reasonable format, to be parsed automatically. If you must use spaces, use quotes."),
     endtime: str = typer.Option(None, "--end", "-end", help="Identify end time. Use any reasonable format, to be parsed automatically. If you must use spaces, use quotes."),
-    plant_name: str = typer.Option(None, "--plantname", "-pn", help = "Define the EDS ZD for your credentials. This must correlate with your idcs point selection(s)."),
+    plant_name: str = typer.Option(None, "--plantname", "-pn", help = "Provide the EDS ZD for your credentials."),
     #workspacename: str = typer.Option(None,"--workspace","-w", help = "Provide the name of the workspace you want to use, for the secrets.yaml credentials and for the timezone config. If a start time is not provided, the workspace queries can checked for the most recent successful timestamp. "),
     print_csv: bool = typer.Option(False,"--print-csv","-p",help = "Print the CSV style for pasting into Excel."),
     step_seconds: int = typer.Option(None, "--step-seconds", help="You can explicitly provide the delta between datapoints. If not, ~400 data points will be used, based on the nice_step() function."), 
@@ -199,47 +214,9 @@ def trend(
     typer.echo(f"iess_list = {iess_list}")
     typer.echo(f"")
 
-    """
-    # Use the retrieved credentials to log in to the API
-    session = EdsClient.login_to_session(
-        api_url=api_credentials.get("url"),
-        username=api_credentials.get("username"),
-        password=api_credentials.get("password")
-    )
-    """
+    # Use the retrieved credentials to log in to the API, including custom session attributes
+    session = EdsClient.login_to_session_with_api_credentials(api_credentials)
 
-    session = None
-    try:
-        session = EdsClient.login_to_session(
-            api_url=api_credentials.get("url"),
-            username=api_credentials.get("username"),
-            password=api_credentials.get("password"),
-            timeout=10 # Add a 10-second timeout to the request
-        )
-    except Timeout:
-        typer.echo(
-            typer.style(
-                "\nConnection to the EDS API timed out. Please check your VPN connection and try again.",
-                fg=typer.colors.RED,
-                bold=True,
-            )
-        )
-        raise typer.Exit(code=1)
-    except EdsLoginException as e:
-        typer.echo(
-            typer.style(
-                f"\nLogin failed for EDS API: {e}",
-                fg=typer.colors.RED,
-                bold=True,
-            )
-        )
-        raise typer.Exit(code=1)
-    
-
-    # Set the session attributes based on the retrieved credentials
-    session.base_url = api_credentials.get("url")
-    session.zd = api_credentials.get("zd")
-    
     points_data = EdsClient.get_points_metadata(session, iess_list=iess_list)
     
     
@@ -271,7 +248,10 @@ def trend(
         # The plot will use this label to draw a separate line for each 'rows'.
         
         attributes = points_data[iess_list[idx]]
-        label = f"{idcs[idx]}, {attributes.get('DESC')}, {attributes.get('UN')}"
+        unit = attributes.get('UN')
+        label = f"{idcs[idx]}, {attributes.get('DESC')}, ({attributes.get('UN')})"
+        #label = f"{idcs[idx]}, {attributes.get('DESC')}"
+        
         #label = idcs[idx]
         
         # The raw from EdsClient.get_tabular_trend() is brought in like this: 
@@ -284,13 +264,14 @@ def trend(
             
             # All data is appended to the *same* data_buffer,
             # but the unique 'label' tells the buffer which series it belongs to.
-            data_buffer.append(label, ts, av)
+            data_buffer.append(label, ts, av, unit)
 
     # Once the loop is done, you can call your show_static function
     # with the single, populated data_buffer.
 
-    if not environment.matplotlib_enabled() or webplot:
+    if not matplotlib_enabled() or webplot:
         from pipeline import gui_plotly_static
+        #from pipeline import gui_plotly_static_backup_06Oct25 as gui_plotly_static
         #gui_fastapi_plotly_live.run_gui(data_buffer)
         gui_plotly_static.show_static(data_buffer)
     else:
@@ -304,7 +285,19 @@ def trend(
             for row in rows:
                 print(f"{helpers.iso(row.get('ts'))},{row.get('value')},")
 
-@app.command(name="configure", help="Configure and store API and database credentials.")
+@app.command()
+def alarm(
+    idcs: list[str] = typer.Argument(None, help="Provide known idcs values to filter the alarms."), # , "--idcs", "-i"
+    export: str = typer.Option(None, "--export", "-e", help = "Export the .")
+    ):
+    """
+    See all current alarms.
+    """
+    typer.echo("Coming soon - print or export alarms. Filter by IDCS values. Designate a specific export path or rely on the default.")
+    if export is None:
+        export_path = Path().cwd() # or
+
+@app.command(name="config", help="Configure and store API and database credentials.")
 def configure_credentials(
     overwrite: bool = typer.Option(False, "--overwrite", "-o", help="Overwrite existing credentials, with confirmation protection."),
     textedit: bool = typer.Option(False, "--textedit", "-t", help = "Open the config file in a text editor instead of using the guided prompt.")
@@ -314,7 +307,7 @@ def configure_credentials(
     """
     if textedit:
         typer.echo(F"Config filepath: {CONFIG_PATH}")
-        environment.open_text_file_in_default_app(CONFIG_PATH)
+        open_text_file_in_default_app(CONFIG_PATH)
         return
             
     typer.echo("")
@@ -375,23 +368,36 @@ def list_workspaces():
 
 @app.command()
 def install(
-    uninstall: bool = typer.Option(False,"--uninstall","-u",help = "Remove the installation artifacts for the current operating system.")
+    uninstall: bool = typer.Option(False,"--uninstall","-un",help = "Remove the installation artifacts for the current operating system."),
+    upgrade: bool = typer.Option(False, "--upgrade", "-up", help = "Uppgrades will be forece, namely shortcut scripts on Termux will be overwritten even if they already exist."),
+    debug: bool = typer.Option(False, "--debug", "-d", help = "Show debugging output and do not actually perform any installation or uninstallation actions.")
 ):
     """
     Windows: Uninstall the registry context-menu item, the launcher BAT, and the AppData folder
     Termux: Remove the scripts from the .shortcuts/ folder.
     """
+
+    if debug:
+        # is_win_exe(debug=True) # inferred, not yet implemented
+        is_pipx(debug=True)
+        is_pyz(debug=True)
+        is_elf(debug=True)
+        return
+    
     if uninstall:
-        if environment.is_windows():
+        if is_windows():
             if typer.confirm("Are you sure you want to uninstall the registry context-menu item, the launcher BAT, and empty out the AppData folder?"):
                 cleanup_windows_install()
-        elif environment.is_termux():
-            pass
-    else:
-        if environment.is_windows():
-            setup_windows_install()
-        elif environment.is_termux():
-            setup_termux_shortcut()
+        elif is_termux():
+            cleanup_termux_install()
+        return
+
+    if is_windows():
+        typer.echo("AppData will be set up explicity and a content menu item will be added to your Registry.")
+        setup_windows_install()
+    elif is_termux():
+        typer.echo("Scripts will now be added to the $HOME/.shortcuts/ directory for launching from the Termux Widget.")
+        setup_termux_install(force=upgrade)
 
 
 @app.command()
@@ -417,12 +423,58 @@ def ping(
         call_ping(url)
 
 @app.command()
-def export():
+def points_export(
+    export_path: str = typer.Argument(None, help = "Provide a specific export path. If not provided, the export will be saved to the current working directory."),
+    plant_name: str = typer.Option(None, "--plantname", "-pn", help = "Provide the EDS ZD for your credentials."),
+    #filter_idcs: str = typer.Option(None,"--idcs", "-i", help="Provide known idcs values to filter the export."), # , "--idcs", "-i"
+):
     """
-    Export a list of all EDS Points.
+    Export a list of all EDS Points. This is specific to the EDS.
     """
-    typer.echo("Coming soon.")
-    #demo_eds_save_point_export()
+    filter_idcs=None # trouble getting multiple points back, suppress for now
+    if plant_name is None:
+        plant_name = get_configurable_plant_name()
+
+
+    if isinstance(plant_name,str):
+        api_credentials = get_eds_api_credentials(plant_name=plant_name)
+
+    if isinstance(plant_name,list):
+        typer.echo("")
+        typer.echo(f"Multiple plant names provided: {plant_name} ")
+        typer.echo("Querying multiple plants at once currently supported.") 
+        typer.echo("Defaulting to use the first name.")
+        api_credentials = get_eds_api_credentials(plant_name=plant_name[0])
+    
+    # Use the retrieved credentials to log in to the API, including custom session attributes
+    typer.echo("Logging in to session...")
+    session = EdsClient.login_to_session_with_api_credentials(api_credentials)
+    
+
+    typer.echo("Retrieving point export...")
+    if filter_idcs is not None:
+        filter_idcs_list = re.split(r'[,\s]+', filter_idcs)
+        idcs_to_iess_suffix = api_credentials.get("idcs_to_iess_suffix")
+        filter_iess = [x+idcs_to_iess_suffix for x in filter_idcs_list]
+        typer.echo(f"filter_iess = {filter_iess}")
+    else:
+        filter_iess = None
+    point_export_decoded_str = EdsClient.get_points_export(session, filter_iess = filter_iess)
+
+    typer.echo("Saving export file...")
+    app_dir_name = f".{get_package_name()}"
+    if export_path is None:
+        data_dir = Path.home() / app_dir_name / "data" 
+        data_dir.mkdir(parents=True, exist_ok=True)
+        now_time_str = TimeManager(TimeManager.now()).as_safe_isoformat_for_filename()
+        export_path = data_dir / f'{plant_name}-export_eds_points_{now_time_str}.txt'
+    try:
+        EdsClient.save_points_export(point_export_decoded_str, export_path = export_path)
+    except Exception as e: # Catch the actual save errors here
+        typer.echo(f"ERROR: Failed to save export file to: {export_path}")
+        typer.echo(f"Details: {e}")
+        return
+    typer.echo(f"\nExport file saved to: \n{export_path}\n")
 
 @app.command()
 def help():
