@@ -20,6 +20,8 @@ except ImportError:
 APP_NAME = get_package_name()
 PACKAGE_NAME = get_package_name() # Used for executable name and AppData folder
 INSTALL_VERSION = get_package_version()
+PACKAGE_ALIAS_EXE = f"{get_package_name()}-exe" # alias for non-pipx binary
+PACKAGE_ALIAS_PYZ = f"{get_package_name()}-pyz"
 INSTALL_VERSION_FILE = "install_version.txt"
 LOG_FILE = "install_log.txt" # New constant for logging
 
@@ -87,6 +89,8 @@ def setup_appdata_dir() -> Path:
     Returns the path to the configuration directory.
     """
     # Use environment variable for robustness
+    if not is_windows():
+        return
     local_appdata = os.environ.get('LOCALAPPDATA')
     if not local_appdata:
         # Fallback using Path.home()
@@ -174,8 +178,10 @@ def setup_windows_install():
     exe_path = get_executable_path()
     if not exe_path:
         # Use simple print for this critical error, as installation won't proceed
-        print("Error: Could not determine running executable path (likely running from source). Aborting Windows setup.", file=sys.stderr)
+        #print("Error: Could not determine running executable path (likely running from source). Aborting Windows setup.", file=sys.stderr)
         return
+    else:
+        print("pipeline.windows_setup.setup_windows_install() ...")
     
     short_path_ref = fr"%LOCALAPPDATA%\{PACKAGE_NAME}"
 
